-- 
-- This file is a part of The Silly Little Oracle Benchmark (a.k.a. SLOB)
--
-- Copyright (c) 1999-2015 Kevin Closson and Kevin Closson d.b.a. Peak Performance Systems
--
-- The Software
-- ------------
-- SLOB is a collection of software, configuration files and documentation (the "Software").
--
-- Use
-- ---
-- Permission is hereby granted, free of charge, to any person obtaining a copy of the Software, to
-- use the Software. The term "use" is defined as copying, viewing, modifying, executing and disclosing
-- information about use of the Software to third parties.
--
-- Redistribution
-- --------------
-- Permission to redistribute the Software to third parties is not granted. The Software           
-- is obtainable from kevinclosson.net/slob. Any redistribution of the Software to third parties
-- requires express written permission from Kevin Closson.
--
-- The copyright notices and permission notices shall remain in all copies of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
-- BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
-- IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-- USE OR OTHER DEALINGS IN THE SOFTWARE.


create or replace directory mydir as '/tmp';
HOST ./mywait

-- SET SERVEROUTPUT OFF ;
SET SERVEROUTPUT ON   ;
SET VERIFY OFF;


DECLARE
v_default_schema_number PLS_INTEGER := '&1';
v_update_pct PLS_INTEGER := '&2';
v_max_loop_iterations PLS_INTEGER := '&3';
v_seconds_to_run PLS_INTEGER := '&4';
v_scale PLS_INTEGER := '&5';
v_work_unit PLS_INTEGER := '&6' ;
v_redo_stress  VARCHAR2(12) := '&7';
v_hot_schema_modulus PLS_INTEGER := '&8';
v_nt_arg  PLS_INTEGER := '&9';
v_do_hotspot   VARCHAR(7) := '&10';
v_hotspot_mb  NUMBER := '&11';
v_hotspot_offset_mb NUMBER := &12;
v_hotspot_modulus  PLS_INTEGER := '&13';

v_sleep_modulus PLS_INTEGER := &14;
v_sleep_min NUMBER := &15;
v_sleep_max NUMBER := &16;

v_hotspot_base NUMBER(15) := ( v_hotspot_offset_mb * 1024 * 1024 ) / 8192 ;
v_hotspot_blocks NUMBER(15) := ( v_hotspot_mb * 1024 * 1024 ) / 8192 ;

v_num_tmp NUMBER := 0;

v_loop_cnt PLS_INTEGER := 0;
v_rowcnt PLS_INTEGER := 0;
v_updates PLS_INTEGER := 0;
v_selects PLS_INTEGER := 0;
v_random_block PLS_INTEGER := 1;
v_tmp PLS_INTEGER;
v_now PLS_INTEGER;
v_brick_wall PLS_INTEGER;

v_begin_time PLS_INTEGER;
v_end_time PLS_INTEGER;
v_total_time PLS_INTEGER;
v_begin_cpu_tm PLS_INTEGER;
v_end_cpu_tm PLS_INTEGER;
v_total_cpu_tm PLS_INTEGER;

v_do_sleeps BOOLEAN := FALSE;
v_loop_control BOOLEAN := FALSE;
v_update_quota BOOLEAN := FALSE;
v_select_only_workload BOOLEAN := FALSE;
v_update_only_workload BOOLEAN := FALSE;
v_do_update BOOLEAN := FALSE;
v_do_hot_schema BOOLEAN := FALSE;
v_stop_immediate BOOLEAN := FALSE;
v_nt BOOLEAN := FALSE;
v_sharing_schema BOOLEAN := FALSE;
v_seed VARCHAR2(50);
v_home_schema_str VARCHAR2(80);
v_scratch VARCHAR2(80) ;

v_cpu_pct NUMBER(6,3);

v_fd1 UTL_FILE.FILE_TYPE;

BEGIN

v_home_schema_str := 'ALTER SESSION SET CURRENT_SCHEMA = user' || v_default_schema_number ;
EXECUTE IMMEDIATE v_home_schema_str; 

v_seed := TO_CHAR(SYSTIMESTAMP,'YYYYDDMMHH24MISSFFFF');
DBMS_RANDOM.seed (val => v_seed);

IF ( v_nt_arg = 1 ) THEN
	v_nt := TRUE ;	
ELSE
	v_nt := FALSE;
 	v_fd1 := UTL_FILE.FOPEN('MYDIR', 'db_stats.out', 'A');
END IF;

IF ( v_hot_schema_modulus != 0 ) THEN 
v_do_hot_schema := TRUE;
END IF;
IF ( v_sleep_modulus != 0 )       THEN 
v_do_sleeps := TRUE;
END IF;
IF ( v_max_loop_iterations > 0 )  THEN 
v_loop_control := TRUE ;
END IF;
IF ( v_update_pct = 0 )           THEN 
v_select_only_workload := TRUE;
END IF;
IF ( v_update_pct = 100 )         THEN 
v_update_only_workload := TRUE;
END IF;

v_update_quota := FALSE ;
v_begin_cpu_tm := DBMS_UTILITY.GET_CPU_TIME();
v_begin_time := DBMS_UTILITY.GET_TIME();
v_now := v_begin_time ;
v_seconds_to_run := v_seconds_to_run * 100 ;
v_brick_wall := v_now + v_seconds_to_run ;

WHILE ( v_now < v_brick_wall AND v_stop_immediate != TRUE )  LOOP

	-- Handle think time:
        IF ( v_do_sleeps = TRUE ) AND ( MOD( v_random_block, v_sleep_modulus ) = 0 ) THEN
		v_num_tmp := ROUND (DBMS_RANDOM.VALUE(v_sleep_min, v_sleep_max) , 2 );
		-- dbms_output.put_line( 'step 1 ' || v_num_tmp  );
               	DBMS_LOCK.SLEEP(v_num_tmp);
        END IF;

	-- If user chose multiple schemas but sharing user1 for contention:
	IF (v_do_hot_schema = TRUE) AND ( MOD(v_loop_cnt, v_hot_schema_modulus) = 0 ) THEN
		EXECUTE IMMEDIATE 'ALTER SESSION SET CURRENT_SCHEMA = user1';
		v_sharing_schema := TRUE ;
	ELSE
		IF ( v_sharing_schema = TRUE ) THEN
			EXECUTE IMMEDIATE v_home_schema_str;
			v_sharing_schema := FALSE;
		END IF;
	END IF;

	-- Handle hot spot
	IF ( v_do_hotspot = 'TRUE' ) AND ( MOD(v_loop_cnt, v_hotspot_modulus ) = 0 ) THEN
 		v_random_block := DBMS_RANDOM.VALUE( v_hotspot_base  + v_work_unit + 1, v_hotspot_base + v_hotspot_blocks) ;
		-- dbms_output.put_line( 'v_loop_cnt ' || v_loop_cnt || ' base ' || (v_hotspot_base  + v_work_unit + 1)  || ' zone ' || v_hotspot_blocks ||  '  Random block ' || v_random_block  );
	ELSE
		v_random_block := DBMS_RANDOM.VALUE(v_work_unit + 1, v_scale) ;
	END IF;

--
-- The following section governs what the SQL execution will be (UPDATE or SELECT)
--

	IF  ( v_select_only_workload = TRUE ) THEN 
		-- handle case where user specified zero pct updates
		v_do_update := FALSE;
		v_update_quota := TRUE ;

	ELSE
		IF ( v_update_only_workload = TRUE ) THEN
			-- handle case where user specified 100% updates
			v_do_update := TRUE;
			v_update_quota := FALSE;
		ELSE			
			IF ( v_update_quota = FALSE ) THEN
				-- Handle case where user specified >0% and < 100% updates
				--
				-- We are doing updates during this run and quota has not been met yet
				-- We still vacilate until update quota has been met

				IF ( MOD(v_random_block, 2) = 0 ) THEN
					v_do_update := TRUE;
				ELSE
					v_do_update := FALSE;
				END IF;
			ELSE
				-- UPDATE quota has been filled, force drain some SELECTs
				v_do_update := FALSE; 
			END IF;
		END IF;
	END IF;

--
-- The type of SQL is now determined...execute it in the following section
--

	IF ( v_do_update != TRUE ) THEN
	--
	-- Do a SELECT statement
	--
		SELECT COUNT(c2) INTO v_rowcnt 
		FROM cf1 
		WHERE ( custid > ( v_random_block - v_work_unit ) ) AND  (custid < v_random_block);
		v_selects := v_selects + 1;		

	ELSE
	--
	-- Do an UPDATE statement
	--
		user1.slobupdate( v_random_block, v_work_unit, v_redo_stress );	
	--
	-- Increment count of UPDATES and set UPDATE quota flag if needed
	--
		v_updates := v_updates + 1;
		IF ( v_updates >= v_update_pct ) THEN
			v_update_quota := TRUE;	
		END IF;
	END IF ;

	--
	-- If this is not a SELECT-only workload, and the global quota is filled,
	-- then reset flags and counters before continuing 
	--

	IF ( v_select_only_workload != TRUE ) AND ( ( v_updates + v_selects ) >=  100 ) THEN
		v_update_quota := FALSE;
		v_updates := 0;
		v_selects := 0;
	END IF;

	v_loop_cnt := v_loop_cnt + 1 ;

        IF ( v_loop_control = TRUE ) AND  ( v_loop_cnt >= v_max_loop_iterations ) THEN
                v_stop_immediate := TRUE ;
        END IF;

	v_now := DBMS_UTILITY.GET_TIME();

END LOOP;

v_end_time := v_now ;
v_now := DBMS_UTILITY.GET_TIME();
v_end_cpu_tm := DBMS_UTILITY.GET_CPU_TIME(); 

v_total_time := v_end_time - v_begin_time ;
v_total_cpu_tm := v_end_cpu_tm - v_begin_cpu_tm  ;
v_cpu_pct := ( v_total_cpu_tm / v_total_time ) * 100 ;
v_scratch := v_default_schema_number || '|' || v_total_time || '|' || v_total_cpu_tm || '|' || v_cpu_pct || '|' ;

if ( v_nt = FALSE ) THEN
	UTL_FILE.PUT_LINE(v_fd1, v_scratch  );
	UTL_FILE.FFLUSH(v_fd1); 
END IF;

END;
/
exit

